/*
    $Id: BinnedProductReaderVPI.java,v 1.4 2007-06-14 17:00:16 ralf Exp $

    Copyright (c) 2006 Brockmann Consult. All rights reserved. Use is
    subject to license terms.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the Lesser GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the BEAM software; if not, download BEAM from
    http://www.brockmann-consult.de/beam/ and install it.
*/
package org.esa.beam.globcolour.visat;

import org.esa.beam.framework.ui.io.BeamFileFilter;
import org.esa.beam.globcolour.dataio.BinnedProductReaderPlugIn;
import org.esa.beam.visat.AbstractProductImportPlugIn;

import javax.swing.*;

/**
 * A VISAT plug-in which installs a new command which lets user import
 * GlobColour Binned products.
 * <p/>
 * The plug-in creates a new menu item in VISAT's file/import menu.
 *
 * @author Ralf Quast
 * @version $Revision: 1.4 $ $Date: 2007-06-14 17:00:16 $
 * @see AbstractProductImportPlugIn
 */
public class BinnedProductReaderVPI extends AbstractProductImportPlugIn {

    /**
     * Constructs this plug-in instance by providing the base class constructor
     * with all required product reader information.
     *
     * @see AbstractProductImportPlugIn
     */
    public BinnedProductReaderVPI() {
        super("org.esa.beam.globcolour.dataio.BinnedProductReader",
              new BeamFileFilter(BinnedProductReaderPlugIn.FORMAT_NAME,
                                 new String[]{BinnedProductReaderPlugIn.FILE_EXTENSION},
                                 "GlobColour Binned Products"), /*I18N*/
                                                                true);
    }

    /**
     * Called by the VISAT {@link AbstractProductImportPlugIn} base class after the plug-in
     * instance has been registered with VISAT's plug-in manager.
     * <p/>
     * This implemenation sets some basic command properties, such as accellerator key and
     * menu text.
     *
     * @see AbstractProductImportPlugIn
     */
    protected void initPlugInImpl() {
        super.initPlugInImpl();
        getCommand().setText("Import GlobColour Binned Product (or Subset)"); /*I18N*/
        getCommand().setShortDescription("Import GlobColour Binned Product (or Subset)"); /*I18N*/
        getCommand().setAccelerator(KeyStroke.getKeyStroke("ctrl B"));
    }
}
