package org.esa.beam.globcolour.visat;

import org.esa.beam.framework.ui.io.BeamFileFilter;
import org.esa.beam.globcolour.dataio.MappedProductReaderPlugIn;
import org.esa.beam.visat.AbstractProductImportPlugIn;

import javax.swing.*;

/**
 * A VISAT plug-in which installs a new command which lets user import
 * GlobColour Mapped products.
 *
 * @author Ralf Quast
 * @version $Revision: 1.5 $ $Date: 2007-06-14 17:13:07 $
 */
public class MappedProductReaderVPI extends AbstractProductImportPlugIn {

    /**
     * Constructs this plug-in instance by providing the base class constructor
     * with all required product reader information.
     *
     * @see AbstractProductImportPlugIn
     */
    public MappedProductReaderVPI() {
        super("org.esa.beam.globcolour.dataio.MappedProductReader",
              new BeamFileFilter(MappedProductReaderPlugIn.FORMAT_NAME,
                                 new String[]{MappedProductReaderPlugIn.FILE_EXTENSION},
                                 "GlobColour Mapped Products"), /*I18N*/
                                                                true);
    }

    /**
     * Called by the VISAT {@link AbstractProductImportPlugIn} base class after the
     * plug-in instance has been registered with VISAT's plug-in manager.
     * <p/>
     * This implemenation sets some basic command properties, such as accellerator
     * key and menu text.
     *
     * @see AbstractProductImportPlugIn
     */
    protected void initPlugInImpl() {
        super.initPlugInImpl();
        getCommand().setText("Import GlobColour Mapped Product (or Subset)"); /*I18N*/
        getCommand().setShortDescription("Import GlobColour Mapped Product (or Subset)"); /*I18N*/
        getCommand().setAccelerator(KeyStroke.getKeyStroke("control M"));
    }
}
